// C Source File
// Created 11/26/00; 2:38:03 PM
// Updated 12/28/06 3:05PM
//  Version 1.3

#define SAVE_SCREEN             // Save/Restore LCD Contents
#define _ti92plus               // Produce .89Z File
#define _ti89

#include <tigcclib.h>
#include "keydef.h"             //Keyboard code defines



void AnimateScreen();           //Interrupt driven water waves, tankmover animation
int BottomSprite(int SpriteID); //ID of bottom sprite in case of stacking 
void ClearGrayScale();          //Clear screen and double buffers
void DrawStatusArea();
void FatalMessage();
short GetKey();                 //Keyboard input routine with APD implemented
void NewFileDialog();           //Handles Calling new file and escape problem
void NewLevelDialog();          //Handles Calling new level and escape problem
void SetupBoard(char mode);     //Setup game board
int StackSprite(int bottom, int top);  //Stack sprite objects
void ToggleTunnelIds();
int TopSprite(int SpriteID);    //ID of top sprite in case of stacking
void WriteSpritePlanes(int r, int c, int SpriteID);  //Write sprites to video planes
void WriteTextPlanes(int x, int y, char *textout, int Attr);

#include "ltank.h"
#include "game.h"
#include "help.h"
#include "editor.h"
#include "leveler.h"
#include "filer.h"
#include "credits.h"
//#include "users.h"

void InitializeIntHandler();
void CentralDispatch();          //Keyboard dispatching for game
void ClearUndoHistory();
void DisplayLevelInfo();
void EditorDispatch();           //Keyboard dispatching for editor
void SetRecord();

DEFINE_INT_HANDLER(MyHandler)
{
  counter++;
  ExecuteHandler(OldInt1);
}

void _main(void)
{
 SCR_RECT stats={{131,1,159,99}};
 SetAlphaLockAddr();
 virtualA = malloc(LCD_SIZE);
 virtualB = malloc(LCD_SIZE);
 virtualC = malloc(LCD_SIZE);
 virtualD = malloc(LCD_SIZE);
 StatsWinL = malloc(BitmapSize(&stats));
 StatsWinD = malloc(BitmapSize(&stats));
 undoRow = malloc(UNDOSIZE);
 undoCol = malloc(UNDOSIZE);
 undoItem = malloc(UNDOSIZE);
 board = malloc(192);
 Tpair = malloc(80);
 memset(Tpair,-1,80);
 BusyTank=False;
 BusyFire=False;
 play=0;
 Shots=0;
 Moves=0;
 Fatal=False;
 InEditor=False;
 AnimateOn=True;
 InitializeFile();
 ClearUndoHistory();
 InitializeIntHandler();
 if (GrayMode(GRAY_ON) || !OSFreeTimer(USER_TIMER))
 {
 	ClearGrayScale();
  OSFreeTimer(USER_TIMER);
  OSRegisterTimer(USER_TIMER,1);
  CreditsPg(1);
  AnimateOn=False;
  GrayOff();      //GrayMode(GRAY_OFF);
  *LevFile = '\0';
  void *kbq = kbd_queue();
  unsigned short key;
  do
  {
  	LoadFile(LevFile,NoFileOpen);
	  while (!OSdequeue(&key,kbq));
  }
  while  (*LevFile == '\0');
  GrayOn();    //GrayMode(GRAY_ON);
  AnimateOn=True;
  ClearGrayScale();
  DrawStatusArea();
  SetupBoard(1);
  CentralDispatch();
 }
 else
  DrawStrXY(5,5,"Failed Gray Mode",A_NORMAL);

 OSFreeTimer(USER_TIMER);
 GrayOff();    //GrayMode(GRAY_OFF);
 SetIntVec(AUTO_INT_1,OldInt1);
 free(virtualD);
 free(virtualC);
 free(virtualB);
 free(virtualA);
 free(StatsWinL);
 free(StatsWinD);
 free(undoRow);
 free(undoCol);
 free(undoItem);
 free(board);
 free(Tpair);
}

void InitializeIntHandler()
{
 OldInt1 = GetIntVec(AUTO_INT_1);
 counter = 0;
 SetIntVec(AUTO_INT_1, MyHandler);
}

void CentralDispatch()
{
 enum Bool ExitLoop;
 short key;
 NewTurnOK=True;
 ExitLoop=False;
 while (!ExitLoop)
 {
  key=GetKey();
  if (!BusyTank && !BusyFire) NewTurnOK=True;
  switch 	(key)
  {	 
   case QUIT_KEY:
    ExitLoop = True; break;
   case CURS_RT:
    if (!BusyTank && !BusyFire) CursHor(1); break;
   case CURS_LT:
    if (!BusyTank && !BusyFire) CursHor(-1); break;
   case CURS_DN:
    if (!BusyTank && !BusyFire) CursVer(1); break;
   case CURS_UP:
    if (!BusyTank && !BusyFire) CursVer(-1); break;
   case X_KEY:
    if (!BusyTank && !BusyFire) Fire(); break;
   case U_KEY:
   case BKSPACE: 
    Undo(); break;
   case PLUS_KEY:
    if (Skip(1))
    {
     SetupBoard(1);
     ClearUndoHistory();
    }
    break;
   case MINUS_KEY:
    if (Skip(-1))
    {
     SetupBoard(1);
     ClearUndoHistory();
    }
    break; 
   case HOME_KEY:
    Skip(0);
    ClearUndoHistory();
    SetupBoard(1);
    BusyTank=0;
    BusyFire=0;
    BusyMove=0;
    break;
   case F1_KEY:
    if (!BusyTank && !BusyFire && !BusyMove)
     HelpScreens(); break;
   case F2_KEY:
    if (!BusyTank && !BusyFire && !BusyMove)
     DisplayLevelInfo(); break;
   case F3_KEY:
    if (!BusyTank && !BusyFire && !BusyMove)
     NewLevelDialog();
    break;
   case F4_KEY:
    if (!BusyTank && !BusyFire && !BusyMove)
     NewFileDialog();
    break;
   case F5_KEY:
    if (!BusyTank && !BusyFire && !BusyMove)
    {
     DisplayPalette(); 
     EditorDispatch();
    }
    break;
   case F6_KEY:
    if (!BusyTank && !BusyFire && !BusyMove)
     ShowCredits(); break;
   case F8_KEY:
    if (!BusyTank && !BusyFire && !BusyMove)
     ClearHallOfFame(LevFile); break;
   case CTRL_F8_KEY:
    if (!BusyTank && !BusyFire && !BusyMove)
     SetGlobals(LevFile); break;
   case CMPLX_I:
    ToggleTunnelIds(); break;
  }
 }
}

void EditorDispatch()
{
 enum Bool ExitLoop;
 short key,changed;
 InEditor=True;
 changed=False;
 OSFreeTimer(USER_TIMER);
 OSRegisterTimer(USER_TIMER,5);
 LockPlace=False;
 board[TankRow][TankCol]=TankUp;
 TankDir=0;
 ExitLoop=False;
 while (!ExitLoop)
 {
  key=GetKey();
  switch (key)
  {	 
   case QUIT_KEY:
    ExitLoop = True;
    CloseEditor();
    AnimateOn=False;
    GrayOff();   //GrayMode(GRAY_OFF);
    EditorFiler(LevFile,changed);
    GrayOn();    //GrayMode(GRAY_ON);
    AnimateOn=True;
    SetupBoard(1);
    break;
   case CURS_RT:
    EdCursHor(1); break;
   case CURS_LT:
    EdCursHor(-1); break;
   case CURS_DN:
    EdCursVer(1); break;
   case CURS_UP:
    EdCursVer(-1); break;
   case DIAM_CLR:
    ClearBoard(); changed=True; break;
   case EQUAL_KEY:
    PlaceObject(); changed=True; break;
   case DIAM_EQUAL:
    if (PaletteRow>0)  
     ToggleLock(); 
    changed=True;
    break;
   case DIAM_UP:
    SelectVer(-1); break;
   case DIAM_RT:
    SelectHor(1); break;
   case DIAM_DN:
    SelectVer(1); break;
   case DIAM_LT:
    SelectHor(-1); break;
   case SHIFT_UP:
    IncTunnelID(1); changed=True; break;
   case SHIFT_DN:
    IncTunnelID(-1); changed=True; break;
  }
 }
 if (changed)
 {
  memset(champ,0,6);
  BestMoves=0;
  BestShots=0;
  GlobalMoves=0;
  GlobalShots=0;
 }
}

short GetKey()
{
 void *kbq = kbd_queue();
 unsigned short TimerDelay=56;
 unsigned short key;
 while (OSdequeue(&key,kbq))
 {
 	if (counter>=TimerDelay && AnimateOn)
 	{
 	 AnimateScreen();
 	 counter=0;
 	}
  if (OSTimerExpired(APD_TIMER))
  {
   off();
   OSTimerRestart(APD_TIMER);
  }
  if ((BusyTank || BusyFire || BusyMove) && OSTimerExpired(USER_TIMER))
  {
   OSTimerRestart(USER_TIMER);
   Animate();
  }
  if (Fatal)
   FatalMessage();
  if (InEditor && OSTimerExpired(USER_TIMER))
  {
   OSTimerRestart(USER_TIMER);
   BlinkCursor();
  }
 } 
 OSTimerRestart(APD_TIMER);
 return key&REPEATMASK;
}

void ClearGrayScale()
{
 memset(virtualA,0,LCD_SIZE);
 memset(virtualB,0,LCD_SIZE);
 memset(virtualC,0,LCD_SIZE);
 memset(virtualD,0,LCD_SIZE);
 memset(GetPlane(DARK_PLANE),0,LCD_SIZE);
 memset(GetPlane(LIGHT_PLANE),0,LCD_SIZE);
}

void FatalMessage()
{	
	void *LightPlane = malloc(LCD_SIZE);
 memcpy(LightPlane,GetPlane(LIGHT_PLANE),LCD_SIZE);
 AnimateOn=False;
 GrayOff();          //GrayMode(GRAY_OFF);
 switch (Fatal)
 {
  case Shot:	
   if (BusyFire==TankLaser)
    DlgMessage("OOPS!","Nailed by your own LASER BEAM! Try again!",BT_OK,BT_NONE);
   else
    DlgMessage("OOPS!","Antitank LASER BEAM got you! Try again!",BT_OK,BT_NONE);
   GrayOn();     //GrayMode(GRAY_ON);
   memcpy(GetPlane(LIGHT_PLANE),LightPlane,LCD_SIZE);  
   Undo();
   break;
  case Sunk:	
   DlgMessage("SPLASH!","You aren't running a SUB! Try again!",BT_OK,BT_NONE);
   GrayOn();    //GrayMode(GRAY_ON);
   memcpy(GetPlane(LIGHT_PLANE),LightPlane,LCD_SIZE);   
   Undo();
   break;
  case CrackedIce:	
   DlgMessage("CRACK! SPLASH","You stopped on THIN ICE! Try again! ",BT_OK,BT_NONE);
   GrayOn();        //GrayMode(GRAY_ON);
   memcpy(GetPlane(LIGHT_PLANE),LightPlane,LCD_SIZE);   
   Undo();
   break;
  case GotFlag:
   if (BestShots+BestMoves==0 || Moves+Shots<BestMoves+BestShots) SetRecord();
   DlgMessage("CONGRATULATIONS!","Proceed to the next level.",BT_OK,BT_NONE);
   GrayOn();        //GrayMode(GRAY_ON);
   if (Skip(1))
   {
    SetupBoard(1);
    ClearUndoHistory();
   }
   else
    memcpy(GetPlane(LIGHT_PLANE),LightPlane,LCD_SIZE);   
   break;
  } 
 free(LightPlane);
 AnimateOn=True;
 Fatal=False;
 BusyTank=False;
 BusyFire=False;
 BusyMove=False;
 TankInTunnel=False;
 ObjectInTunnel=False;
 WriteSpritePlanes(TankRow,TankCol,TankDir);
}

void SetRecord()
{
	BestShots=Shots;
	BestMoves=Moves;
	if (GlobalMoves+GlobalShots==0)
 {
 	GlobalMoves=Moves;
 	GlobalShots=Shots;
 }
	PromptChamp();
	WriteRecord(LevFile);
}

void ToggleTunnelIds()
{
	unsigned i;
	ShowId=!ShowId;
 for (i=0;i<=19;i++)
 {
	 if (Tpair[i][0]>-1)
	 	switch (ShowId)
	 	{
	 		case True:
  	  WriteTunnelNumber(Tpair[i][0],Tpair[i][1],i+80); break;
	   case False:
	    WriteSpritePlanes(Tpair[i][0],Tpair[i][1],Tunnel); break;
	  }
	 if (Tpair[i][2]>-1)
	 	switch (ShowId)
	 	{
	 		case True:
	    WriteTunnelNumber(Tpair[i][2],Tpair[i][3],i+80); break;
	   case False:
	    WriteSpritePlanes(Tpair[i][2],Tpair[i][3],Tunnel); break;
	  }
	}
}

void NewLevelDialog()
{
 unsigned char curlevel;
 void *LightPlane = malloc(LCD_SIZE);
 memcpy(LightPlane,GetPlane(LIGHT_PLANE),LCD_SIZE);
 AnimateOn=False;
 GrayOff();       //GrayMode(GRAY_OFF);
 curlevel=Level;
 if (LevelList(LevFile)==1)
 {
  GrayOn();    //GrayMode(GRAY_ON);
  AnimateOn=True;
  SetupBoard(1);
  ClearUndoHistory();
 }
 else
 {
  GrayOn();          //GrayMode(GRAY_ON);
  memcpy(GetPlane(LIGHT_PLANE),LightPlane,LCD_SIZE);  
  Level=curlevel;
  AnimateOn=True;
 }
  free(LightPlane);
}

void NewFileDialog()
{
 void *LightPlane = malloc(LCD_SIZE);
 memcpy(LightPlane,GetPlane(LIGHT_PLANE),LCD_SIZE);
 AnimateOn=False;
 GrayOff();         //GrayMode(GRAY_OFF);
 if (LoadFile(LevFile,FileIsOpen)==1)
 {
  GrayOn();         //GrayMode(GRAY_ON);
  AnimateOn=True;
  SetupBoard(1);
  ClearUndoHistory();
  Level=0;
 }
 else
 {
  GrayOn();          //GrayMode(GRAY_ON);
  memcpy(GetPlane(LIGHT_PLANE),LightPlane,LCD_SIZE);  
  AnimateOn=True;
 }
 free(LightPlane);
}

void DisplayLevelInfo()
{
 char done;
 WINDOW w;
 char num[]="        ";   //8 spaces
 void *LightPlane = malloc(LCD_SIZE);
 memcpy(LightPlane,GetPlane(LIGHT_PLANE),LCD_SIZE);
 done=0;
 AnimateOn=False;
 GrayOff();          //GrayMode(GRAY_OFF);
 WinOpen(&w,MakeWinRect(15,9,145,97),WF_SAVE_SCR);
 WinActivate(&w);
 WinFont(&w,F_4x6);
 WinMoveTo(&w,0,10);
 WinLineRel(&w,130,0);
 WinMoveTo(&w,0,47);
 WinLineRel(&w,130,0);
 WinMoveTo(&w,0,73);
 WinLineRel(&w,130,0);
 WinMoveTo(&w,63,47);
 WinLineRel(&w,0,26);
 WinStrXY(&w,9,3,"LASER TANK LEVEL INFORMATION");
 WinStrXY(&w,11,15,"File Name:");
 WinStrXY(&w,50,15,LevFile);
 WinStrXY(&w,5,23,"Level Name:");
 WinStrXY(&w,50,23,title);
 WinStrXY(&w,22,31,"Author:");
 WinStrXY(&w,50,31,author);
 WinStrXY(&w,13,39,"Difficulty:");
 WinStrXY(&w,50,39,&dif[difficulty*8]);
 WinStrXY(&w,10,50,"Global");
 WinStrXY(&w,13,58,"Shots:");
 WinStrXY(&w,10,66,"Moves:");
 sprintf(num,"%u",GlobalShots);
 WinStrXY(&w,36,58,num);
 sprintf(num,"%u",GlobalMoves);
 WinStrXY(&w,36,66,num);
 WinStrXY(&w,76,50,champ);
 WinStrXY(&w,79,58,"Shots:");
 WinStrXY(&w,76,66,"Moves:");
 sprintf(num,"%u",BestShots);
 WinStrXY(&w,102,58,num);
 sprintf(num,"%u",BestMoves);
 WinStrXY(&w,102,66,num);
 DrawStaticButton(&w,BT_CANCEL,35);
 while (!done)
 {
  if (GetKey()==ESC_KEY) done=1;
 }
 WinClose(&w);
 GrayOn();           //GrayMode(GRAY_ON);
 memcpy(GetPlane(LIGHT_PLANE),LightPlane,LCD_SIZE);  
 free(LightPlane);
 AnimateOn=True;
}

void WriteSpritePlanes(int r, int c, int spriteID)
{
 int vpA,vpB,vpC,vpD,BitMask;
 
 vpA=TopSprite(spriteID);
 BitMask=0;
 if (vpA>=TankUp && vpA<=CanonLt)
  BitMask=vpA+47;
 if (vpA>=SlideMirrorUR && vpA<=RotateMirrorDL)
  BitMask=vpA+46;
 if (board[r][c]>=80 && board[r][c]<=99)
  BitMask=0;
 if (vpA>=47 && vpA <=52)
  BitMask=2;
 switch (vpA)
 {
  case TankMoverUp: vpB=vpA+1;vpC=vpA+2;vpD=vpA+3; break;
  case TankMoverRt: vpB=vpA+1;vpC=vpA+2;vpD=vpA+3; break;
  case TankMoverDn: vpB=vpA+1;vpC=vpA+2;vpD=vpA+3; break;
  case TankMoverLt: vpB=vpA+1;vpC=vpA+2;vpD=vpA+3; break;
  case Water: vpB=vpA+1;vpC=vpA+2;vpD=vpA+3; break;
  default: vpB=vpC=vpD=vpA;
 }
 Sprite8(tx+c*wid,ty+r*wid,8,DarkSprite[BitMask],GetPlane(DARK_PLANE),SPRT_AND);
 Sprite8(tx+c*wid,ty+r*wid,8,DarkSprite[vpA],GetPlane(DARK_PLANE),SPRT_OR);
 Sprite8(tx+c*wid,ty+r*wid,8,LightSprite[BitMask],GetPlane(LIGHT_PLANE),SPRT_AND);
 Sprite8(tx+c*wid,ty+r*wid,8,LightSprite[vpA],GetPlane(LIGHT_PLANE),SPRT_OR);
 Sprite8(tx+c*wid,ty+r*wid,8,LightSprite[BitMask],virtualA,SPRT_AND);
 Sprite8(tx+c*wid,ty+r*wid,8,DarkSprite[vpA],virtualA,SPRT_OR);
 Sprite8(tx+c*wid,ty+r*wid,8,LightSprite[BitMask],virtualB,SPRT_AND);
 Sprite8(tx+c*wid,ty+r*wid,8,DarkSprite[vpB],virtualB,SPRT_OR);
 Sprite8(tx+c*wid,ty+r*wid,8,LightSprite[BitMask],virtualC,SPRT_AND);
 Sprite8(tx+c*wid,ty+r*wid,8,DarkSprite[vpC],virtualC,SPRT_OR);
 Sprite8(tx+c*wid,ty+r*wid,8,LightSprite[BitMask],virtualD,SPRT_AND);
 Sprite8(tx+c*wid,ty+r*wid,8,DarkSprite[vpD],virtualD,SPRT_OR);
}

void WriteTextPlanes(int x, int y, char *textout, int Attr)
{
	PortSet(virtualA,239,127);
	DrawStrXY(x,y,textout,Attr);
 PortSet(virtualB,239,127);
	DrawStrXY(x,y,textout,Attr);
 PortSet(virtualC,239,127);
	DrawStrXY(x,y,textout,Attr);
 PortSet(virtualD,239,127);
	DrawStrXY(x,y,textout,Attr);
 SetPlane(DARK_PLANE);
	DrawStrXY(x,y,textout,Attr);
 SetPlane(LIGHT_PLANE);
	DrawStrXY(x,y,textout,Attr);
}  
 
int TopSprite(int spriteID)
{
 int ID;
 ID=spriteID;
 if (spriteID>=80)
 {
  if (spriteID<=99)        //Numbered Tunnels  80-99 	
   ID=Tunnel;
  else
   switch((spriteID-100)/10)
   {
    case 0:                //object on SunkenBlock  
     ID=spriteID-90;break;
    case 1:                //object on TankMoverUp
     ID=spriteID-100;break; 
    case 2:                //object on TankMoverRt
     ID=spriteID-110;break; 
    case 3:                //object on TankMoverDn
     ID=spriteID-120;break; 
    case 4:                //object on TankMoverLt
     ID=spriteID-130;break; 
    case 5:                //object on Flag
     ID=spriteID-140;break;           
    case 6:                //object on Ice
     ID=spriteID-150;break;
    case 7:                //object on ThinIce
     ID=spriteID-160;break;
    case 8:                //object on Tunnel Entrance
     ID=spriteID-170;
   }
 }
 return ID;
}

int BottomSprite(int spriteID)
{
 int ID;
 ID=Background;
 if (spriteID<=SunkenBlock || spriteID>=Tunnel)
 {
  if (spriteID<=99)
   ID=spriteID;
  else
   if (spriteID>99)
    switch ((spriteID-100)/10)
    {
     case 0:               //object on SunkenBlock  
      ID=SunkenBlock;break;
     case 1:               //object on TankMoverUp
      ID=TankMoverUp;break; 
     case 2:               //object on TankMoverRt
      ID=TankMoverRt;break; 
     case 3:               //object on TankMoverDn
      ID=TankMoverDn;break; 
     case 4:               //object on TankMoverLt
      ID=TankMoverLt;break; 
     case 5:               //object on Flag
      ID=Flag;break;
     case 6:               //object on Ice
      ID=Ice;break;
     case 7:               //object on ThinIce
      ID=ThinIce;break;
     case 8:               //object on Tunnel Entrance
      ID=LastTunnel;
    }
 }
 return ID;
}

int StackSprite(int bottom, int top)
{
 int Stacked;
 Stacked = top;
 switch (bottom)
 {
  case SunkenBlock:Stacked=90+top;break;
  case TankMoverUp:Stacked=100+top;break;
  case TankMoverRt:Stacked=110+top;break;
  case TankMoverDn:Stacked=120+top;break;
  case TankMoverLt:Stacked=130+top;break;
  case Flag:Stacked=140+top;break;
  case Ice:Stacked=150+top;break;
  case ThinIce:Stacked=160+top;break;
  case Water:
   if (top==MoveableBlock)
    Stacked=SunkenBlock;
   else
    Stacked=Water;		       
   break;
  default:
   if (bottom>=80 && bottom<=99)
   {
   	Stacked=170+top;
   	LastTunnel=bottom;
   }
 }
 return Stacked;
}

void SetupBoard(char mode)
{
 int r,c;
 char lev[4]={"   \0"};
 if (mode==1)
 {
  Moves=0;
  Shots=0;
  memset(Tpair,-1,80);
 }
 TankInTunnel=False;
 ShowId=False;
 Backtrack=False;
 ObjectInTunnel=False;
 SetPlane(LIGHT_PLANE);
 BitmapPut(131,1,StatsWinL,&FullScr,A_REPLACE);
 SetPlane(DARK_PLANE);
 BitmapPut(131,1,StatsWinD,&FullScr,A_REPLACE);
 PortSet(virtualA,239,127);
 BitmapPut(131,1,StatsWinD,&FullScr,A_REPLACE);
 PortSet(virtualB,239,127);
 BitmapPut(131,1,StatsWinD,&FullScr,A_REPLACE);
 PortSet(virtualC,239,127);
 BitmapPut(131,1,StatsWinD,&FullScr,A_REPLACE);
 PortSet(virtualD,239,127);
 BitmapPut(131,1,StatsWinD,&FullScr,A_REPLACE);
 sprintf(lev,"%03u",Level);
 FontSetSys(F_4x6);
 WriteTextPlanes(140,85,lev,A_REPLACE);
 UpdateStats(Up);
 for (c=0;c<16;c++)
  for (r=0;r<12;r++)
  {
   if (board[r][c]>99)      //stacked objects
   {
    WriteSpritePlanes(r,c,BottomSprite(board[r][c]));
    WriteSpritePlanes(r,c,TopSprite(board[r][c]));
   }
   if (board[r][c]>=TankUp && board[r][c]<=RotateMirrorUL)
    WriteSpritePlanes(r,c,0);
   WriteSpritePlanes(r,c,board[r][c]);
   if (mode==1 && board[r][c]>=TankUp && board[r][c]<=TankLt) 
   {
    TankRow=r;
    TankCol=c;
    TankDir=board[r][c];
    board[r][c]=Background;
   }
   if (mode>=0 && board[r][c]>=80 && board[r][c]<= 99)  //tunnel objects
   {
    if (Tpair[board[r][c]-80][0]==-1)
    {
     Tpair[board[r][c]-80][0]=r;
     Tpair[board[r][c]-80][1]=c;
    }
    else
    {
     Tpair[board[r][c]-80][2]=r;
     Tpair[board[r][c]-80][3]=c;
    }
   }
  }
 WriteSpritePlanes(TankRow,TankCol,TankDir); 
}

void DrawStatusArea()
{
	int r,c;
 SCR_RECT Window1 = {{135,5,156,18}};
 SCR_RECT Window2 = {{135,45,156,52}};
 SCR_RECT Window3 = {{135,63,156,70}};
 SCR_RECT Window4 = {{138,83,153,90}};
 SCR_RECT Window5 = {{140,24,151,35}};
 SCR_RECT stat = {{131,1,159,99}};
 for (c=1;c<=6;c++)
 {
 	switch (c)
 	{
 		case 1:
 		 PortSet(virtualA,239,127); break;
 		case 2:
 		 PortSet(virtualB,239,127); break;
 		case 3:
 		 PortSet(virtualC,239,127); break;
 		case 4:
 		 PortSet(virtualD,239,127); break;
 		case 5:
    SetPlane(DARK_PLANE);break;
 		case 6:
    SetPlane(LIGHT_PLANE);
  }
  DrawClipRect(MakeWinRect(1,1,130,98),&FullScr,A_NORMAL);
  DrawClipRect(MakeWinRect(133,1,158,98),&FullScr,A_NORMAL);
 }
 tx=134;
 for (c=0;c<3;c++)
  for (r=0;r<12;r++)
   WriteSpritePlanes(r,c,Brick);
 FontSetSys(F_4x6);
 for (c=1;c<=6;c++)
 {
 	switch (c)
 	{
 		case 1:
 		 PortSet(virtualA,239,127); break;
 		case 2:
 		 PortSet(virtualB,239,127); break;
 		case 3:
 		 PortSet(virtualC,239,127); break;
 		case 4:
 		 PortSet(virtualD,239,127); break;
 		case 5:
    SetPlane(DARK_PLANE);break;
 		case 6:
    SetPlane(LIGHT_PLANE);
  }
  ScrRectFill(&Window1,&FullScr,A_REVERSE); 
  ScrRectFill(&Window2,&FullScr,A_REVERSE); 
  ScrRectFill(&Window3,&FullScr,A_REVERSE); 
  ScrRectFill(&Window4,&FullScr,A_REVERSE); 
  ScrRectFill(&Window5,&FullScr,A_REVERSE);
  DrawStrXY(136,6,"Laser",A_NORMAL);
  DrawStrXY(137,13,"Tank",A_NORMAL);  
  DrawStrXY(137,38,"shots",A_REVERSE);
  DrawStrXY(135,56,"moves",A_REVERSE);
  DrawStrXY(136,76,"level",A_REVERSE);
  DrawStrXY(133,93,"F1 Help",A_REVERSE);
 }
 WriteSpritePlanes(3,1,TankUp);
 SetPlane(LIGHT_PLANE);
 BitmapGet(&stat,StatsWinL);
 SetPlane(DARK_PLANE);
 BitmapGet(&stat,StatsWinD);
 tx=2;
}	

void ClearUndoHistory()
{
 memset(undoRow,255,UNDOSIZE);
 memset(undoCol,255,UNDOSIZE);
 memset(undoItem,255,UNDOSIZE);
}

void AnimateScreen()
{
 static int flipflop=0;
 switch (flipflop)
 {
  case 0:
   memcpy(GetPlane(DARK_PLANE),virtualA,LCD_SIZE); break;
  case 1:
   memcpy(GetPlane(DARK_PLANE),virtualB,LCD_SIZE); break;
  case 2:
   memcpy(GetPlane(DARK_PLANE),virtualC,LCD_SIZE); break;
  case 3:
   memcpy(GetPlane(DARK_PLANE),virtualD,LCD_SIZE); break;
 }
 flipflop++;
 if (flipflop>3) flipflop=0;
}
